@@ -1,21 +1,8 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
-from typing import Any
-from typing import Optional
 from typing import TextIO
-from typing import Union
 
 import cProfile
 import json
@@ -30,9 +17,10 @@ from gi.repository import Gio
 from gi.repository import GLib
 from nbxmpp.const import ConnectionProtocol
 from nbxmpp.const import ConnectionType
-from nbxmpp.http import HTTPRequest
+from nbxmpp.protocol import JID
 from packaging.version import Version as V
 
+from gajim import IS_PORTABLE
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import ged
@@ -41,27 +29,28 @@ from gajim.common import passwords
 from gajim.common.cert_store import CertificateStore
 from gajim.common.client import Client
 from gajim.common.commands import ChatCommands
-from gajim.common.dbus import file_manager
-from gajim.common.dbus import logind
+from gajim.common.events import AccountCreated
 from gajim.common.events import AccountDisabled
 from gajim.common.events import AccountDisconnected
 from gajim.common.events import AccountEnabled
+from gajim.common.events import AccountRemoved
 from gajim.common.events import AllowGajimUpdateCheck
+from gajim.common.events import DBMigrationError
 from gajim.common.events import GajimUpdateAvailable
 from gajim.common.events import SignedIn
-from gajim.common.helpers import from_one_line
-from gajim.common.helpers import get_global_show
-from gajim.common.helpers import get_random_string
-from gajim.common.settings import LegacyConfig
+from gajim.common.file_transfer_manager import FileTransfer
+from gajim.common.file_transfer_manager import FileTransferManager
 from gajim.common.settings import Settings
-from gajim.common.storage.archive import MessageArchiveStorage
+from gajim.common.storage.archive.storage import MessageArchiveStorage
 from gajim.common.storage.cache import CacheStorage
 from gajim.common.storage.draft import DraftStorage
-from gajim.common.storage.events import EventStorage
+from gajim.common.storage.events.storage import EventStorage
+from gajim.common.task_manager import PulseManager
 from gajim.common.task_manager import TaskManager
-from gajim.common.util.http import create_http_request
-from gajim.plugins.pluginmanager import PluginManager
-from gajim.plugins.repository import PluginRepository
+from gajim.common.util.text import from_one_line
+from gajim.common.util.text import get_random_string
+
+log = logging.getLogger('gajim.c.application')
 
 
 class CoreApplication(ged.EventHelper):
@@ -69,40 +58,49 @@ class CoreApplication(ged.EventHelper):
         ged.EventHelper.__init__(self)
         self._profiling_session = None
 
-    def _init_core(self) -> None:
+    def _init_core(self) -> bool:
         # Create and initialize Application Paths & Databases
         app.app = self
         app.print_version()
         app.detect_dependencies()
+        app.init_process_pool()
         configpaths.create_paths()
 
-        passwords.init()
-
         app.settings = Settings()
         app.settings.init()
 
-        app.config = LegacyConfig()
+        passwords.init()
+
         app.commands = ChatCommands()
 
-        app.storage.cache = CacheStorage()
-        app.storage.cache.init()
+        try:
+            app.storage.cache = CacheStorage()
+            app.storage.cache.init()
 
-        app.storage.events = EventStorage()
-        app.storage.events.init()
+            app.storage.events = EventStorage()
+            app.storage.events.init()
 
-        app.storage.archive = MessageArchiveStorage()
-        app.storage.archive.init()
+            app.storage.archive = MessageArchiveStorage()
+            app.storage.archive.init()
+        except Exception as error:
+            app.ged.raise_event(DBMigrationError(exception=error))
+            log.exception('Failed to init storage')
+            return False
 
         app.storage.drafts = DraftStorage()
 
         app.cert_store = CertificateStore()
         app.task_manager = TaskManager()
+        app.pulse_manager = PulseManager()
 
-        from gajim.common.call_manager import CallManager
-        app.call_manager = CallManager()
+        app.ftm = FileTransferManager()
 
-        from gajim.common.preview import PreviewManager
-        app.preview_manager = PreviewManager()
+        # from gajim.common.call_manager import CallManager
+        # app.call_manager = CallManager()
+
+        if app.is_installed("GST"):
+            from gajim.gtk.audio_player import AudioPlayer
+            app.audio_player = AudioPlayer()
 
         self._network_monitor = Gio.NetworkMonitor.get_default()
         self._network_monitor.connect('notify::network-available',
@@ -111,21 +109,21 @@ class CoreApplication(ged.EventHelper):
 
         if sys.platform in ('win32', 'darwin'):
             GLib.timeout_add_seconds(20, self._check_for_updates)
-        else:
-            logind.enable()
-
-        file_manager.init()
 
         for account in app.settings.get_active_accounts():
             app.connections[account] = Client(account)
             app.to_be_removed[account] = []
-            app.nicks[account] = app.settings.get_account_setting(account,
-                                                                  'name')
+            app.nicks[account] = app.get_default_nick(account)
+
+        from gajim.plugins.pluginmanager import PluginManager
+        from gajim.plugins.repository import PluginRepository
 
         app.plugin_manager = PluginManager()
         app.plugin_manager.init_plugins()
         app.plugin_repository = PluginRepository()
 
+        app.commands.init()
+
         for client in app.get_clients():
             client.get_module('Roster').load_roster()
 
@@ -135,9 +133,11 @@ class CoreApplication(ged.EventHelper):
             ('signed-in', ged.CORE, self._on_signed_in),
         ])
 
+        return True
+
     @property
     def _log(self) -> logging.Logger:
-        return app.log('gajim.application')
+        return app.log('app')
 
     def _core_command_line(self, options: GLib.VariantDict) -> None:
         if options.contains('cprofile'):
@@ -212,12 +212,11 @@ class CoreApplication(ged.EventHelper):
         self._profiling_session.disable()
         self._log.info('End profiling')
         ps = pstats.Stats(self._profiling_session)
-        ps = ps.sort_stats(SortKey.CUMULATIVE)
+        ps = ps.sort_stats(SortKey.TIME)
         ps.print_stats()
 
-    def start_shutdown(self, *args: Any, **kwargs: Any) -> None:
-        app.app.systray.shutdown()
-
+    def _start_shutdown(self) -> None:
+        self._log.info("Start Shutdown")
         accounts_to_disconnect: dict[str, Client] = {}
 
         for client in app.get_clients():
@@ -225,13 +224,15 @@ class CoreApplication(ged.EventHelper):
                 accounts_to_disconnect[client.account] = client
 
         if not accounts_to_disconnect:
-            self._quit_app()
+            self._shutdown_core()
+            self._shutdown_complete()
             return
 
         def _on_disconnect(event: AccountDisconnected) -> None:
             accounts_to_disconnect.pop(event.account)
             if not accounts_to_disconnect:
-                self._quit_app()
+                self._shutdown_core()
+                self._shutdown_complete()
                 return
 
         app.ged.register_event_handler('account-disconnected',
@@ -239,32 +240,36 @@ class CoreApplication(ged.EventHelper):
                                        _on_disconnect)
 
         for client in accounts_to_disconnect.values():
-            client.change_status('offline', kwargs.get('message', ''))
+            client.change_status('offline', '')
 
     def _shutdown_core(self) -> None:
+        self._log.info("Shutdown core")
         # Commit any outstanding SQL transactions
+        app.process_pool.shutdown(cancel_futures=True)
         app.storage.archive.cleanup_chat_history()
         app.storage.cache.shutdown()
         app.storage.archive.shutdown()
+        app.settings.save()
         app.settings.shutdown()
-        self.end_profiling()
-        logind.shutdown()
 
-    def _quit_app(self) -> None:
-        self._shutdown_core()
+        self.end_profiling()
+        app.app.systray.shutdown()
+        configpaths.cleanup_temp()
+
+    def _shutdown_complete(self) -> None:
+        self._log.info("Shutdown complete")
 
     @staticmethod
     def _show_warnings() -> None:
         import traceback
         import warnings
-        os.environ['GAJIM_LEAK'] = 'true'
 
-        def warn_with_traceback(message: Union[Warning, str],
+        def warn_with_traceback(message: Warning | str,
                                 category: type[Warning],
                                 filename: str,
                                 lineno: int,
-                                _file: Optional[TextIO] = None,
-                                line: Optional[str] = None) -> None:
+                                _file: TextIO | None = None,
+                                line: str | None = None) -> None:
 
             traceback.print_stack(file=sys.stderr)
             sys.stderr.write(warnings.formatwarning(message, category,
@@ -295,6 +300,10 @@ class CoreApplication(ged.EventHelper):
         if not app.settings.get('check_for_update'):
             return
 
+        if sys.platform == 'win32' and app.is_ms_store():
+            # Gajim updates are handled by MS Store directly
+            return
+
         now = datetime.now()
         last_check = app.settings.get('last_update_check')
         if not last_check:
@@ -309,33 +318,44 @@ class CoreApplication(ged.EventHelper):
 
     def check_for_gajim_updates(self) -> None:
         self._log.info('Checking for Gajim updates')
-        request = create_http_request()
-        request.send('GET', 'https://gajim.org/current-version.json',
-                     callback=self._on_update_response)
 
-    def _on_update_response(self, request: HTTPRequest) -> None:
+        app.ftm.http_request(
+            'GET',
+            'https://gajim.org/current-version.json',
+            callback=self._on_update_response,
+        )
 
-        if not request.is_complete():
-            self._log.warning('Error while performing gajim update check: %s',
-                              request.get_error(), request.get_status())
+    def _on_update_response(self, obj: FileTransfer) -> None:
+        try:
+            result = obj.get_result()
+        except Exception as error:
+            self._log.warning(
+                'Error while performing gajim update check: %s', error)
             return
 
         now = datetime.now()
         app.settings.set('last_update_check', now.strftime('%Y-%m-%d %H:%M'))
 
-        data = json.loads(request.get_data())
-        latest_version = data['current_version']
+        try:
+            data = json.loads(result.content)
+            latest_version = data['current_version']
+            setup_url = data['current_win_setup']
+            if IS_PORTABLE:
+                setup_url = data['current_win_portable_setup']
+        except Exception:
+            self._log.exception("Unable to perform update check")
+            return
 
         if V(latest_version) > V(app.version):
-            app.ged.raise_event(GajimUpdateAvailable(version=latest_version))
+            app.ged.raise_event(GajimUpdateAvailable(version=latest_version,
+                                                     setup_url=setup_url))
             return
 
         self._log.info('Gajim is up to date')
 
     def create_account(self,
                        account: str,
-                       username: str,
-                       domain: str,
+                       address: JID,
                        password: str,
                        proxy_name: str | None,
                        custom_host: tuple[str,
@@ -344,20 +364,22 @@ class CoreApplication(ged.EventHelper):
                        anonymous: bool = False
                        ) -> None:
 
-        account_label = f'{username}@{domain}'
         if anonymous:
-            username = 'anon'
-            account_label = f'anon@{domain}'
+            address = address.new_with(localpart='anon')
 
-        config: dict[str, Union[str, int, bool]] = {
-            'name': username,
+        if not address.localpart:
+            raise ValueError('Username must be set')
+
+        account_label = str(address)
+
+        config: dict[str, str | int | bool] = {
+            'address': str(address),
             'resource': f'gajim.{get_random_string(8)}',
             'account_label': account_label,
-            'hostname': domain,
             'anonymous_auth': anonymous,
         }
 
-        if proxy_name is not None:
+        if proxy_name:
             config['proxy'] = proxy_name
 
         use_custom_host = custom_host is not None
@@ -373,8 +395,11 @@ class CoreApplication(ged.EventHelper):
         for opt, value in config.items():
             app.settings.set_account_setting(account, opt, value)  # pyright: ignore  # noqa: E501
 
-        # Password module depends on existing config
-        passwords.save_password(account, password)
+        if not anonymous:
+            # Password module depends on existing config
+            passwords.save_password(account, password)
+
+        app.ged.raise_event(AccountCreated(account=account))
 
     def enable_account(self, account: str) -> None:
         app.connections[account] = Client(account)
@@ -383,7 +408,7 @@ class CoreApplication(ged.EventHelper):
             app.connections[account])
 
         app.to_be_removed[account] = []
-        app.nicks[account] = app.settings.get_account_setting(account, 'name')
+        app.nicks[account] = app.get_default_nick(account)
         app.settings.set_account_setting(account, 'active', True)
 
         app.ged.raise_event(AccountEnabled(account=account))
@@ -399,8 +424,6 @@ class CoreApplication(ged.EventHelper):
 
         app.get_client(account).cleanup()
         del app.connections[account]
-        if account in app.interface.instances:
-            del app.interface.instances[account]
         del app.nicks[account]
         del app.to_be_removed[account]
 
@@ -412,23 +435,22 @@ class CoreApplication(ged.EventHelper):
         # Delete password must be before del_per() because it calls set_per()
         # which would recreate the account with defaults values if not found
         passwords.delete_password(account)
+        app.storage.archive.remove_account(account)
         app.settings.remove_account(account)
         app.app.remove_account_actions(account)
 
+        app.ged.raise_event(AccountRemoved(account=account))
+
     def _on_signed_in(self, event: SignedIn) -> None:
         client = app.get_client(event.account)
-        app.storage.archive.insert_jid(client.get_own_jid().bare)
-
         if client.get_module('MAM').available:
             client.get_module('MAM').request_archive_on_signin()
 
     def change_status(self,
                       status: str,
-                      account: Optional[str] = None
+                      account: str | None = None
                       ) -> None:
 
-        status = get_global_show()
-
         if account is not None:
             self._change_status(account, status)
             return
