@@ -12,7 +12,7 @@ from gajim.common.i18n import _
 from gajim.common.modules.contacts import GroupchatContact
 
 from gajim.gtk.builder import get_builder
-from gajim.gtk.contact_name_widget import ContactNameWidget
+from gajim.gtk.contact_name_entry import ContactNameEntry
 from gajim.gtk.groupchat_affiliation import GroupchatAffiliation
 from gajim.gtk.groupchat_blocks import GroupchatBlocks
 from gajim.gtk.groupchat_config import GroupchatConfig
@@ -24,7 +24,7 @@ from gajim.gtk.omemo_trust_manager import OMEMOTrustManager
 from gajim.gtk.sidebar_switcher import SideBarMenuItem
 from gajim.gtk.sidebar_switcher import SideBarSwitcher
 from gajim.gtk.structs import AccountJidParam
-from gajim.gtk.widgets import GajimAppWindow
+from gajim.gtk.window import GajimAppWindow
 
 
 class GroupchatDetails(GajimAppWindow):
@@ -34,8 +34,6 @@ class GroupchatDetails(GajimAppWindow):
             name="GroupchatDetails",
             title=_("Group Chat Details"),
             default_height=600,
-            add_window_padding=False,
-            header_bar=False,
         )
 
         self.account = contact.account
@@ -137,7 +135,6 @@ class GroupchatDetails(GajimAppWindow):
     def _on_disco_info_update(
         self, _contact: GroupchatContact, _signal_name: str
     ) -> None:
-
         self._groupchat_info.set_info_from_contact(self._contact)
 
     def _on_avatar_update(self, _contact: GroupchatContact, _signal_name: str) -> None:
@@ -209,5 +206,5 @@ class GroupchatDetails(GajimAppWindow):
         self._ui.configuration_box.append(config)
         self._switcher.set_item_visible("config", True)
 
-    def _on_contact_name_updated(self, _widget: ContactNameWidget, name: str) -> None:
+    def _on_contact_name_updated(self, _widget: ContactNameEntry, name: str) -> None:
         self._sidebar_page.set_title(name)
